from datetime import datetime

from cpl_core.database import TableABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.model.server import Server


class ShortRoleName(TableABC):
    def __init__(
        self,
        name: str,
        discord_role_id: int,
        server: Server,
        created_at: datetime = None,
        modified_at: datetime = None,
        id=0,
    ):
        self._id = id
        self._name = name
        self._discord_role_id = discord_role_id
        self._server = server

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._id

    @property
    def name(self) -> str:
        return self._name

    @name.setter
    def name(self, value: str):
        self._name = value

    @property
    def role_id(self) -> int:
        return self._discord_role_id

    @role_id.setter
    def role_id(self, value: int):
        self._discord_role_id = value

    @property
    @ServiceProviderABC.inject
    def role_name(self, bot: DiscordBotServiceABC) -> str:
        guild = bot.get_guild(self._server.discord_id)
        return guild.get_role(self.role_id).name

    @property
    def server(self) -> Server:
        return self._server

    @staticmethod
    def get_select_all_string() -> str:
        return str(
            f"""
            SELECT * FROM `ShortRoleNames`;
        """
        )

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `ShortRoleNames`
            WHERE `Id` = {id};
        """
        )

    @staticmethod
    def get_select_by_server_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `ShortRoleNames`
            WHERE `ServerId` = {id};
        """
        )

    @property
    def insert_string(self) -> str:
        return str(
            f"""
            INSERT INTO `ShortRoleNames` (
                `Name`, `DiscordRoleId`, `ServerId`
            ) VALUES (
                '{self._name}',
                {self._discord_role_id}
                {self._server.id},
            );
        """
        )

    @property
    def udpate_string(self) -> str:
        return str(
            f"""
            UPDATE `ShortRoleNames`
            SET `Name` = '{self._name}',
            `DiscordRoleId` = {self._discord_role_id},
            `ServerId` = {self._server.id}
            WHERE `Id` = {self._id};
        """
        )

    @property
    def delete_string(self) -> str:
        return str(
            f"""
            DELETE FROM `ShortRoleNames`
            WHERE `Id` = {self._id};
        """
        )
